/*
 * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */

package com.openexchange.guard.exceptions;

import com.openexchange.exception.Category;
import com.openexchange.exception.DisplayableOXExceptionCode;
import com.openexchange.exception.OXException;
import com.openexchange.exception.OXExceptionFactory;
import com.openexchange.exception.OXExceptionStrings;

/**
 * {@link GuardAuthExceptionCodes}
 *
 * @author <a href="mailto:ioannis.chouklis@open-xchange.com">Ioannis Chouklis</a>
 */
public enum GuardAuthExceptionCodes implements DisplayableOXExceptionCode {

    LOCKOUT("Account lockout.", GuardAuthExceptionMessages.LOCKOUT_MSG, CATEGORY_PERMISSION_DENIED, 1),
    BAD_PASSWORD("Bad password.", GuardAuthExceptionMessages.BAD_PASSWORD_MSG, CATEGORY_PERMISSION_DENIED, 2),
    MISSING_SESSION("No valid session found.", GuardAuthExceptionMessages.MISSING_SESSION_MSG,CATEGORY_PERMISSION_DENIED,3),
    FORBIDDEN("The operation is not permitted.", GuardAuthExceptionMessages.FORBIDDEN_MSG,CATEGORY_PERMISSION_DENIED,4),
    AUTH_MISSING("Authentication parameter missing.", GuardAuthExceptionMessages.AUTH_MISSING_MSG, CATEGORY_ERROR, 5),
    TOKEN_ERROR("Unable to get session token.", GuardAuthExceptionMessages.TOKEN_ERROR_MSG, CATEGORY_ERROR, 6),
    NOT_LOGGED_IN("User does not appear logged in.", GuardAuthExceptionMessages.NOT_LOGGED_IN, CATEGORY_ERROR, 7),
    RECOVERY_MISSING("No recovery available", GuardAuthExceptionMessages.RECOVERY_MISSING, CATEGORY_ERROR, 8),
    ;

    private String message;
    private String displayMessage;
    private Category category;
    private int number;

    private GuardAuthExceptionCodes(String message, Category category, int number) {
        this(message, null, category, number);
    }

    private GuardAuthExceptionCodes(String message, String displayMessage, Category category, int number) {
        this.message = message;
        this.displayMessage = displayMessage != null ? displayMessage : OXExceptionStrings.MESSAGE;
        this.category = category;
        this.number = number;
    }

    @Override
    public String getPrefix() {
        return "GRD-AUTH";
    }

    @Override
    public int getNumber() {
        return number;
    }

    @Override
    public String getMessage() {
        return message;
    }

    @Override
    public String getDisplayMessage() {
        return displayMessage;
    }

    @Override
    public Category getCategory() {
        return category;
    }

    @Override
    public boolean equals(OXException e) {
        return OXExceptionFactory.getInstance().equals(this, e);
    }

    /**
     * Creates a new {@link OXException} instance pre-filled with this code's attributes.
     *
     * @return The newly created {@link OXException} instance
     */
    public OXException create() {
        return OXExceptionFactory.getInstance().create(this, new Object[0]);
    }

    /**
     * Creates a new {@link OXException} instance pre-filled with this code's attributes.
     *
     * @param args The message arguments in case of printf-style message
     * @return The newly created {@link OXException} instance
     */
    public OXException create(Object... args) {
        return OXExceptionFactory.getInstance().create(this, (Throwable) null, args);
    }

    /**
     * Creates a new {@link OXException} instance pre-filled with this code's attributes.
     *
     * @param cause The optional initial cause
     * @param args The message arguments in case of printf-style message
     * @return The newly created {@link OXException} instance
     */
    public OXException create(Throwable cause, Object... args) {
        return OXExceptionFactory.getInstance().create(this, cause, args);
    }
}
